SystemOrganization addCategory: #SeasideJ!

WAKom subclass: #KomSeasideJ
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SeasideJ'!

KomSeasideJ class
	instanceVariableNames: 'isPatched'!

!KomSeasideJ class methodsFor: 'private' stamp: 'mu 1/5/2006 15:48'!
guessIsPatched
	"KomSeasideJ guessIsPatched"
	| versionString |
	versionString := SystemVersion current version.
	(versionString = 'Squeakland 3.8-05'
			and: [SystemVersion current highestUpdate > 500])
		ifTrue: [^ true].
	^ false! !

!KomSeasideJ class methodsFor: 'class initialization' stamp: 'mu 1/5/2006 16:48'!
initialize
	"KomSeasideJ initialize"
	super initialize.
	isPatched := nil! !

!KomSeasideJ class methodsFor: 'accessing' stamp: 'mu 1/5/2006 15:48'!
isPatched
	isPatched ifNil: [isPatched _ self guessIsPatched].
	^isPatched! !

!KomSeasideJ class methodsFor: 'service' stamp: 'mu 9/3/2005 00:21'!
startOn: aPort
	super startOn: aPort.
	SeasideSettings port: aPort! !

!KomSeasideJ class methodsFor: 'service' stamp: 'mu 9/3/2005 00:22'!
stop
	super stop.
	SeasideSettings port: nil
	! !

!KomSeasideJ methodsFor: 'actions' stamp: 'mu 1/5/2006 15:34'!
convertResponse: aResponse
	| komResponse responseStream |
	aResponse ifNil: [^ HttpResponse fromString: 'Request handling aborted; reload to retry'].

	komResponse :=  HttpResponse new.

	aResponse cookies do:
		[:assoc | komResponse setCookieName: assoc key value: assoc valueWithExpiry path: assoc path].
		

	self outerHeadersFrom: aResponse headers.
	aResponse headers associationsDo: [:assoc | komResponse fieldAt: assoc key put: assoc value].
	(HttpResponse classPool at: #StatusCodes) associationsDo:
		[:assoc |
		assoc value key = aResponse status ifTrue: [komResponse status: assoc key]].
	

	" conversion to UTF-8 "
	responseStream := aResponse.
	(aResponse contentType upTo: $/) = 'text' ifTrue:
		[responseStream := MultiByteBinaryOrTextStream on: WideString new encoding: #utf8.
		responseStream nextPutAll: aResponse contents.
		responseStream binary reset].
	
	komResponse contents: responseStream contents readStream.
	komResponse contentType: aResponse contentType.

	

	^ komResponse! !

!KomSeasideJ methodsFor: 'private' stamp: 'mu 1/5/2006 01:02'!
innerValuesDictFrom: rawDictionary 
	| dic |
	dic := rawDictionary copy.
	rawDictionary
		keysAndValuesDo: [:k :v | 
			| done | 
			done := false.
			v isString
				ifTrue: [dic at: k put: v convertFromHtmlInput.
					done := true].
			(done not
					and: [v isCollection])
				ifTrue: ["for Form"
					dic
						removeKey: k
						ifAbsent: [].
					(self innerValuesFrom: v)
						do: [:each | dic at: k put: each].
					done := true].
			(done not
					and: [v isMemberOf: WAFile])
				ifTrue: ["for File upload"
					v fileName: v fileName convertFromHtmlInput.
					dic at: k put: v.
					done := true].
			done
				ifFalse: ["default"
					dic at: k put: v]].
	^ dic! !

!KomSeasideJ methodsFor: 'private' stamp: 'mu 8/17/2005 17:56'!
innerValuesFrom: rawCollection 
	
	^rawCollection
		collect: [:each | each convertFromHtmlInput].
! !

!KomSeasideJ methodsFor: 'testing' stamp: 'mu 1/5/2006 15:49'!
isPatched
	^self class isPatched! !

!KomSeasideJ methodsFor: 'private' stamp: 'mu 9/6/2005 00:16'!
outerContentDispositionValueFrom: aString 
	| str wStr former latter |
	aString isWideString
		ifFalse: [^ aString].
	str := aString readStream.
	wStr := '' writeStream.
	[str atEnd
		or: [str peek = $=]]
		whileFalse: [wStr nextPut: str next].
	former := wStr contents.
	latter := str skip: 1; upToEnd.
	latter := latter encodeForHTTP.
	^ former , '*=''''', latter! !

!KomSeasideJ methodsFor: 'private' stamp: 'mu 9/5/2005 20:31'!
outerHeadersFrom: headers 
	headers ifEmpty: [^ headers].
	headers
		do: [:keyAndValue | 
			| key value | 
			key := keyAndValue key asLowercase.
			value := keyAndValue value.
			(key = 'location') ifTrue: [keyAndValue
						value: (self outerLocationValueFrom: value)].
			(key = 'content-disposition')
				ifTrue: [keyAndValue
						value: (self outerContentDispositionValueFrom: value)]].
	^ headers! !

!KomSeasideJ methodsFor: 'private' stamp: 'mu 9/5/2005 23:24'!
outerHeaderValueFrom: aString
	aString isWideString ifTrue: [^aString encodeForHTTP].
	^aString! !

!KomSeasideJ methodsFor: 'private' stamp: 'mu 9/5/2005 23:23'!
outerLocationValueFrom: aString
	^self outerValueFrom: aString! !

!KomSeasideJ methodsFor: 'private' stamp: 'mu 9/5/2005 23:23'!
outerValueFrom: aString
	aString isWideString ifTrue: [^aString encodeForHTTP].
	^aString! !

!KomSeasideJ methodsFor: 'actions' stamp: 'mu 1/5/2006 15:49'!
process: komRequest
	|request response komResponse |
	self log: komRequest.
	request := self convertRequest: komRequest.
	self isPatched ifFalse: [request :=  self requestInnerCoded: request].
	response := self handleRequest: request.
	komResponse := self convertResponse: response.
	response release.
	^ komResponse! !

!KomSeasideJ methodsFor: 'private' stamp: 'mu 1/5/2006 16:49'!
requestInnerCoded: aWARequest 
	| req methodString urlString headers fields cookies |
	req := aWARequest.
	methodString := req method.
	urlString := req url.
	headers :=  req headers.
	fields := self innerValuesDictFrom: req fields.
	cookies :=  req cookies.
	^ WARequest
		method: methodString
		url: urlString
		headers: headers
		fields: fields
		cookies: cookies! !

!WideString methodsFor: '*seasideJ' stamp: 'mu 8/1/2005 19:25'!
prettyPrintHtmlOn: aStream indentLevel: aNumber

	aStream nextPutAll: (self convertToHtmlOutput).
! !

!WideString methodsFor: '*seasideJ' stamp: 'mu 8/1/2005 19:26'!
printHtmlOn: aStream

	aStream nextPutAll: (self convertToHtmlOutput).
! !

Object subclass: #SeasideSettings
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SeasideJ'!

SeasideSettings class
	instanceVariableNames: 'outerConverter innerConverter port'!

!SeasideSettings class methodsFor: 'accessing' stamp: 'mu 8/1/2005 19:23'!
innerConverter
	innerConverter ifNil: [innerConverter := (Smalltalk at: #UTF8TextConverter ifAbsent:[]) ifNotNilDo: [:c | c new]].
	^innerConverter! !

!SeasideSettings class methodsFor: 'accessing' stamp: 'mu 8/1/2005 19:21'!
innerConverter: aTextConverter
	innerConverter _ aTextConverter! !

!SeasideSettings class methodsFor: 'accessing' stamp: 'mu 8/1/2005 19:24'!
outerConverter
	outerConverter ifNil: [outerConverter := (Smalltalk at: #UTF8TextConverter ifAbsent:[]) ifNotNilDo: [:c | c new]].
	^outerConverter! !

!SeasideSettings class methodsFor: 'accessing' stamp: 'mu 8/1/2005 19:22'!
outerConverter: aTextConverter
	outerConverter := aTextConverter! !

!SeasideSettings class methodsFor: 'accessing' stamp: 'mu 9/3/2005 00:20'!
port
	^port! !

!SeasideSettings class methodsFor: 'private' stamp: 'mu 9/3/2005 00:20'!
port: anInteger
	port _ anInteger! !

!String methodsFor: '*seasideJ' stamp: 'mu 8/1/2005 19:28'!
convertFromHtmlInput

	^ self convertFromWithConverter: SeasideSettings innerConverter
! !

!String methodsFor: '*seasideJ' stamp: 'mu 8/1/2005 19:27'!
convertToHtmlOutput

	^ self convertToWithConverter: SeasideSettings outerConverter
! !

!String methodsFor: '*seasideJ-override' stamp: 'mu 1/13/2006 00:09'!
unescapePercentsWithTextEncoding: encodingName
	"decode string including %XX form"
	| ans c asciiVal pos oldPos specialChars encodedStream converter |
	encodedStream _ ReadWriteStream on: String new.
	converter _ TextConverter newForEncoding: encodingName.
	ans _ WriteStream on: String new.
	oldPos _ 1.
	specialChars _ '+%' asCharacterSet.

	[pos _ self indexOfAnyOf: specialChars startingAt: oldPos. pos > 0]
	whileTrue: [
		ans nextPutAll: (self copyFrom: oldPos to: pos - 1).
		c _ self at: pos.
		(c = $% and: [pos + 2 <= self size]) 
			ifTrue: [
				encodedStream resetToStart.
				[c = $% ] whileTrue: [
	
				asciiVal _ (self at: pos+1) asUppercase digitValue * 16 +
							(self at: pos+2) asUppercase digitValue.
	
				asciiVal > 255 ifTrue: [^self].	"not really an escaped string".
					encodedStream nextPut: (Character value: asciiVal).
	
				pos _ pos + 3.
					(pos <= self size) 
						ifTrue:  [c _ self at: pos]
						ifFalse:  [c _ nil].
				].
				encodedStream position: 0.
				[(c _ converter nextFromStream: encodedStream) notNil ] whileTrue: [
								ans nextPut: c].
				oldPos _ pos.
			]
			ifFalse: [ (c = $+) ifTrue: [ans nextPut: Character space] ifFalse: [ans nextPut: c].
					oldPos _ pos + 1
			].
	].
	(oldPos <= self size) ifTrue: [
		ans nextPutAll: (self copyFrom: oldPos to: self size) ].

	^ ans contents! !
KomSeasideJ initialize!
